import path from "path";
import { scanFiles, extractMatchesFromFiles, writeJsonToFile, copyFileToDir } from "@wemt/core-node/utils";

// 默认配置
const defaultConfig = {
  scan: {
    path: ["src"], // 要扫描的文件路径
    ext: [".vue"], // 要扫描的文件扩展名
    match: [/['"]([^'"]*fa-[^'"]*)['"]/g], // 匹配规则（如图标类名前缀）
  },
  output: {
    path: "src/assets/fontawesome/svg", // 输出目录
    name: "fortawesome.json", // 输出提取后的json列表
  },
  icon: {
    path: [
      "node_modules/@fortawesome/fontawesome-free/svgs/regular",
      "node_modules/@fortawesome/fontawesome-free/svgs/solid",
      "node_modules/@fortawesome/fontawesome-free/svgs/brands",
    ], // 图标配置文件路径
    ext: [".svg"], // 图标配置文件扩展名
    prefix: "fa-", // 图标前缀
  },
};

async function extractClasses(config) {
  console.log("Starting to scan files...");

  // 使用scanFiles扫描文件
  const files = await scanFiles(config.scan);
  console.log(`Found ${files.length} ${config.scan.ext.join(", ")} files`);

  // 使用extractMatchesFromFiles提取匹配的类名
  const matches = extractMatchesFromFiles(files, config.scan);

  // 收集所有唯一的类名
  const faClasses = new Set();
  Object.values(matches).forEach((fileMatches) => {
    fileMatches.forEach((match) => faClasses.add(match.replace(config.icon.prefix, "")));
  });

  const svg = await scanFiles(config.icon);
  console.log(`Found ${svg.length} ${config.icon.ext.join(", ")} files`);

  // 创建图标名称到路径的映射
  const svgMap = {};
  svg.forEach((svgPath) => {
    const fileName = path.basename(svgPath, ".svg");
    svgMap[fileName] = svgPath;
  });

  // 创建结果对象
  const result = {};
  faClasses.forEach((faClass) => {
    if (svgMap[faClass]) {
      result[faClass] = svgMap[faClass];
      const outputPath = path.join(config.output.path, `${faClass}.svg`);
      copyFileToDir(svgMap[faClass], outputPath);
      console.log(`Copied ${faClass}.svg to ${outputPath}`);
    }
  });

  console.log(`\nSummary:`);
  console.log(`- Total files processed: ${Object.keys(matches).length}`);
  console.log(`- Total unique matches found: ${Object.keys(result).length}`);
  return result;
}

// Vite 插件
export default function wemtFontawesomePurge(userConfig = {}) {
  // 合并用户配置和默认配置
  const config = {
    scan: { ...defaultConfig.scan, ...userConfig.scan },
    output: { ...defaultConfig.output, ...userConfig.output },
    icon: { ...defaultConfig.icon, ...userConfig.icon },
  };

  return {
    name: "wemt-vite-plugin-fontawesome-purge",
    // 在构建开始时执行
    async buildStart() {
      try {
        console.log("Starting pattern matching extraction...");
        const classes = await extractClasses(config);
        if (Object.keys(classes).length > 0) {
          const outputPath = path.join(config.output.path, config.output.name);
          await writeJsonToFile(outputPath, classes, { indent: 2 });
          console.log(`\nExtracted data has been saved to: ${outputPath}`);
        } else {
          console.log("No matches found in the scanned files.");
        }
      } catch (error) {
        console.error("Error occurred:", error);
      }
    },
  };
}
